"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const readline = require("readline");
const semver = require("semver");
const chalk_1 = require("chalk");
const project_1 = require("./project");
const server_1 = require("./server");
process.title = "Framer Project";
// Stop when `stdin` closes. This make sure we quit when Framer exits unexpectedly.
process.stdin.resume();
process.stdin.on("end", () => {
    process.exit();
});
const usage = () => {
    // tslint:disable-next-line:no-console
    console.log(chalk_1.default `
{greenBright Usage:}

{bold serve} <project.framerfx> [--port=<port>] [--staticPath=<path>]

    Run an HTTP server with auto reload. Use --staticPath to serve the
    specified path under /static/.

{bold install} <project.framerfx>

    Install dependencies of the project.

{bold add} <project.framerfx> <package>

    Add a dependency to the project.

{bold build} <project.framerfx>

    Build the JavaScript file for the project.

{bold publish} <project.framerfx> [--major] [--public]

    Publish the project to the private store (use --public for public).
    Will bump the version automatically. Use --major for major version
    bump.

{greenBright Global options:}

    --api=<url>          API endpoint to use
    --author=<name>      Set author if there is none [HACK!]
    --cachePath=<path>   Use a custom cache path
    --registry=<domain>  Registry to use (for publishing and @framer/*)
    --token=<token>      Used for authentication
`);
};
const commands = {
    add(project, argv) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Consider what should happen for .framerx files.
            const packageName = argv._[2];
            if (!packageName) {
                error("Missing package name");
                return;
            }
            yield project.add(packageName, false);
        });
    },
    build(project, argv) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Consider what should happen for .framerx files.
            yield project.install();
            yield project.build();
        });
    },
    install(project, argv) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Consider what should happen for .framerx files.
            yield project.install();
        });
    },
    publish(project, argv) {
        return __awaiter(this, void 0, void 0, function* () {
            const isPublic = argv.public !== undefined;
            const pkg = project.packageJson;
            if (!pkg.name || !pkg.version) {
                error("Missing package information");
                return;
            }
            const version = semver.inc(pkg.version, argv.major !== undefined ? "major" : "minor");
            yield project.install();
            yield project.buildDist();
            let promptText = `Ready to publish `;
            promptText += chalk_1.default `{bold ${pkg.name}} v{bold ${version}} `;
            if (isPublic) {
                promptText += chalk_1.default `({greenBright public store}). `;
            }
            else {
                promptText += chalk_1.default `({yellowBright private store}). `;
            }
            promptText += `Continue? `;
            const answer = yield prompt(promptText);
            if (!answer.match(/^(y|yes)/i)) {
                process.stdout.write("Aborting...");
                process.exit();
            }
            yield project.publish(version, pkg.name, undefined, {}, isPublic ? "public" : "restricted");
        });
    },
    serve(project, argv) {
        return __awaiter(this, void 0, void 0, function* () {
            const chosenPort = yield project.serve(portRange(argv.port || argv.p));
            process.title += ` ${chosenPort}`;
            yield project.install();
            project.watch();
            yield project.updatePackageInfo();
            return true; // don’t exit
        });
    },
};
const main = () => __awaiter(this, void 0, void 0, function* () {
    const argv = minimist(process.argv.slice(2));
    const command = argv._[0];
    const config = {
        projectPath: argv._[1],
        staticPath: argv.staticPath,
        cachePath: argv.cachePath || "/tmp/framer",
        token: argv.token,
        api: argv.api,
        author: argv.author,
        proxyURL: argv.proxyURL,
        proxyDocumentURL: argv.proxyDocumentURL,
        proxyImageBaseURL: argv.proxyImageBaseURL,
    };
    if (!config.projectPath) {
        if (command !== "serve") {
            // All other commands require a project directory.
            usage();
            process.exit();
        }
        // Directory not available yet – the project will be created later.
        const server = new server_1.Server(config);
        const chosenPort = yield server.start(portRange(argv.port || argv.p));
        process.title += ` ${chosenPort}`;
        return; // don’t exit
    }
    let project;
    try {
        project = yield project_1.Project.load(config);
    }
    catch (e) {
        return error(e.message);
    }
    let keepRunning = false;
    if (command in commands) {
        keepRunning = yield commands[command](project, argv);
    }
    else {
        usage();
    }
    if (!keepRunning) {
        process.exit();
    }
});
main().catch(e => error(e.message));
// Because Raven is slow, install it after the server started (we’re not using the Middleware)
require("raven")
    .config("https://05cdf9fe436f4b058c893701e0a577c8@sentry.io/1253080")
    .install();
function error(message) {
    // tslint:disable-next-line:no-console
    console.log(chalk_1.default `\n{redBright ERROR: ${message}}`);
    usage();
    return process.exit();
}
function portRange(base = 4567, count = 100) {
    return [base, base + count];
}
function prompt(question) {
    return __awaiter(this, void 0, void 0, function* () {
        const rl = readline.createInterface(process.stdin, process.stdout);
        return new Promise(resolve => rl.question(question, resolve));
    });
}
